extrapconvfunction [t,y,h] = IMPExtrapLog(h,y0,T,lambda)
% IMPEXTRAPLOG compute the numerical solution of the logistic
% differential equation with an extrapolated mid-point rule.
%
%   Name: 
%
% [T,Y,H] = IMPEXTRAPLOG(H,Y0,T,LAMBDA) solve the logistic differential
% equation y' = LAMBDA*y(1-y) on the time interval T = [t0,tend] with 
% initial value y(t0) = y0 with an extrapolated mid-point rule and step
% size (at most) H. As output, the time grid T, the solution Y and (possibly
% altered) step size are returned. The returned value of the step size is
% necessary for the convergence analysis.
%
% Example:
% [t,y,h] = IMPExtrapLog(0.05,0.02,[0,1],10);
%
% See also IMPEXTRAPLOGCONV.
  
  N = ceil((T(2)-T(1))/h); % number of steps
  h = (T(2)-T(1))/N; % adjust step size to fit interval length
  t = linspace(T(1),T(2),N+1); % time grid
  y = nan(1,N+1); % solution
  y(1) = y0; % set initial value

  for j=1:N
    
    % Solution at t(j)+h computed with a step of size h
    y1 = IMPLogStep(y(j),h,lambda);
    
    % Solution at t(j)+h computed with a step of size h/2
    y2 = IMPLogStep(y(j),h/2,lambda);
    y2 = IMPLogStep(y2,h/2,lambda);
    
    % Extrapolated solution
    y(j+1) = -y1/3 + 4*y2/3;
    
  end

  return
  
function y1 = IMPLogStep(y0,h,lambda)
  % IMPLOGSTEP compute the solution Y1 of the logistic differential
  % equation with parameter value LAMBDA after a step H starting
  % at the initial value Y0 using the implicit mid-point rule
  
  z = h*lambda;
  y1 = -y0 + 2*( -(1-z/2) + sqrt( (1-z/2)^2 + 2*z*y0 ) )/z;
  
  return
  
